@@ -60,11 +60,11 @@
 #'   combined with `model_code = TRUE`, which adds the Stan model code to the
 #'   return object.
 #' @param ... For `dynamite()`, additional arguments to [rstan::sampling()] or
-#'  [cmdstanr::sample()], such as `chains` and `cores` (`parallel_chains` in
-#'  `cmdstanr`). For `summary()`, additional arguments to
-#'  [dynamite::as.data.frame.dynamitefit()]. For `print()`, further arguments
-#'  to the print method for tibbles (see [tibble::formatting]). Not used for
-#'  `formula()`.
+#'   [cmdstanr::sample()], such as `chains` and `cores` (`parallel_chains` in
+#'   `cmdstanr`). For `summary()`, additional arguments to
+#'   [dynamite::as.data.frame.dynamitefit()]. For `print()`, further arguments
+#'   to the print method for tibbles (see [tibble::formatting]). Not used for
+#'   `formula()`.
 #' @return `dynamite` returns a `dynamitefit` object which is a list containing
 #'   the following components:
 #'
@@ -206,15 +206,12 @@ dynamite <- function(dformula, data, time, group = NULL,
     }
     data[[group]] <- 1L
   }
-  data_name <- attr(dformula, "data_name")
-  if (is.null(data_name)) {
-    d <- match.call()$data
-    data_name <- ifelse_(
-      is.symbol(d),
-      deparse1(d),
-      ""
-    )
-  }
+  d <- match.call()$data
+  data_name <- ifelse_(
+    is.symbol(d),
+    deparse1(d),
+    "NULL"
+  )
   data <- parse_data(dformula, data, group, time, verbose)
   dformula <- parse_past(dformula, data, group, time)
   dformulas <- parse_lags(dformula, data, group, time, verbose)
@@ -582,8 +579,9 @@ formula.dynamitefit <- function(x, ...) {
       "override = FALSE, ",
       "df = ", spline_def$bs_opts$df, ", ",
       "degree = ", spline_def$bs_opts$degree, ", ",
-      "lb_tau = ", spline_def$lb, ", ",
-      "noncentered = ", spline_def$noncentered, ")"
+      "lb_tau = c(", cs(spline_def$lb), "), ",
+      "noncentered = c(", cs(spline_def$noncentered), ")",
+      ")"
     )
   )
   lfactor_def <- attr(x$dformulas$stoch, "lfactor")
@@ -598,7 +596,8 @@ formula.dynamitefit <- function(x, ...) {
       "responses = ", lfactor_resp, ", ",
       "noncentered_psi = ", lfactor_def$noncentered_psi, ", ",
       "nonzero_lambda = c(", cs(lfactor_def$nonzero_lambda), "), ",
-      "correlated = ", lfactor_def$correlated, ")"
+      "correlated = ", lfactor_def$correlated,
+      ")"
     )
   )
   random_spec_def <- attr(x$dformulas$stoch, "random_spec")
@@ -607,7 +606,8 @@ formula.dynamitefit <- function(x, ...) {
     paste0(
       "random_spec(",
       "correlated = ", random_spec_def$correlated, ", ",
-      "noncentered = ", random_spec_def$noncentered, ")"
+      "noncentered = ", random_spec_def$noncentered,
+      ")"
     )
   )
   str2lang(
